package com.load.service.impl;

import com.load.domain.MqttServerModel;
import com.load.domain.Result;
import com.load.domain.VinIp;
import com.load.domain.req.VehicleConnectionReq;
import com.load.service.LoadBalanceService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import static com.load.util.ECSTool.runEcsInstance;

/**
 * 服务实现类
 *
 * @author YunFei·Du
 * @ClassName: ConnectServiceImpl
 * @Description: 服务实现类
 * @CreateTime: 2024/5/27 09:22
 */
@Service
@Slf4j
public class LoadBalanceServiceImpl implements LoadBalanceService {


    @Autowired
    private RedisTemplate<String, String> redisTemplate;


    @Override
    public Result<MqttServerModel>  getConnect(VehicleConnectionReq vehicleConnectionReq) {

        log.error("车辆连接请求:{}",vehicleConnectionReq.toString());


        //判断是否有游标key --count
        if (redisTemplate.hasKey("count")){

            Integer count = Integer.valueOf(redisTemplate.opsForValue().get("count"));
            if (count == 100){
                redisTemplate.opsForValue().set("count",String.valueOf(0));
            }else {
                redisTemplate.opsForValue().set("count",String.valueOf(count + 1));
            }

            //根据游标count获取服务IP
            String ip = redisTemplate.opsForList().index("ipList", count);
            //关联车辆和服务
            this.insertVinIp(new VinIp (vehicleConnectionReq.getVehicleVin(),ip));
            //响应信息
            return Result.success(new MqttServerModel("tcp://"+ip+":1883","test1"));
        }else {
            redisTemplate.opsForValue().set("count",String.valueOf(1));
            //根据游标count获取服务IP
            String ip = redisTemplate.opsForList().index("ipList", 0);
            //关联车辆和服务
            this.insertVinIp(new VinIp(vehicleConnectionReq.getVehicleVin(),ip));
            //响应信息
            return Result.success(new MqttServerModel("tcp://"+ip+":1883","test1"));
        }
    }

    /**
     * 添加车辆和ip都按redis
     * @param vinIp
     */
    private void insertVinIp(VinIp vinIp) {
        redisTemplate.opsForHash().put("vinIp",vinIp.getVin(),vinIp.getIp());
    }

    @SneakyThrows
    @Override
    public void createConnect() {
        runEcsInstance("cn-zhangjiakou", "lt-8vbepqjihmawbkqcwkcm");
    }




}
