package com.load.util;

import com.aliyun.ecs20140526.Client;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.tea.TeaException;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;
import lombok.extern.log4j.Log4j2;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**

  *  ECS实例工具类
  * @author YunFei.Du
  * @date  9:30 2024/5/28
 */
@Log4j2
public class ECSTool {


    /**
     * 创建ECS客户端
     * @return ECS客户端实例
     * @throws Exception 如果配置信息不正确或网络问题，将抛出异常
     * 注意：此方法用于演示，实际使用时请替换为安全的鉴权方式，如STS
     * 更多鉴权访问方式参考：https://help.aliyun.com/document_detail/378657.html
     */
    public static Client createClient() throws Exception {
        // 工程代码泄露可能会导致 AccessKey 泄露，并威胁账号下所有资源的安全性。以下代码示例仅供参考。
        Config config = new Config()
                .setAccessKeyId("LTAI5tPTk3MFkmCGBbnQgmrM")
                .setAccessKeySecret("q7rLjxrI0SLBXlvNT4VmYcHCNCY2p6");
        // Endpoint 请参考 https://api.aliyun.com/product/Ecs
        config.endpoint = "ecs.cn-zhangjiakou.aliyuncs.com";
        return new Client(config);
    }

    /**
     * 创建并运行ECS实例
     * @param regionId 地域ID
     * @param launchTemplateId 启动模板ID
     * @throws Exception 如果调用API时发生错误，将抛出异常
     */
    public static void runEcsInstance(String regionId, String launchTemplateId) throws Exception {
        // 创建ECS客户端
        Client client = ECSTool.createClient();
        // 设置运行实例的请求参数
        RunInstancesRequest runInstancesRequest = new RunInstancesRequest()
                .setRegionId(regionId)
                .setLaunchTemplateId(launchTemplateId);
        // 创建运行选项
        RuntimeOptions runtime = new RuntimeOptions();

        try {
            // 调用API运行实例
            client.runInstancesWithOptions(runInstancesRequest, runtime);
        } catch (Exception error) {
            // 处理API调用过程中出现的异常
            System.out.println(error.getMessage());
            if (error instanceof TeaException) {
                // 处理特定类型的异常，如TeaException
                TeaException teaError = (TeaException) error;
                System.out.println(teaError.getData().get("Recommend")); // 打印诊断推荐链接
                com.aliyun.teautil.Common.assertAsString(teaError.getMessage()); // 断言错误信息
            } else {
                // 处理其他类型的异常
                System.out.println(error.getMessage());
            }
        }
    }

    /**
     * 销毁实例
     */
    public static void runEcsRemove(String instanceId) throws Exception {
        // 创建ECS客户端
        Client client = ECSTool.createClient();
        DeleteInstanceRequest deleteInstanceRequest = new DeleteInstanceRequest ()
                .setInstanceId(instanceId);
        RuntimeOptions runtime = new RuntimeOptions();
        try {
            // 复制代码运行请自行打印 API 的返回值
            client.deleteInstanceWithOptions(deleteInstanceRequest, runtime);
        } catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println(error.getMessage());
            // 诊断地址
            System.out.println(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        } catch (Exception _error) {
            TeaException error = new TeaException(_error.getMessage(), _error);
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println(error.getMessage());
            // 诊断地址
            System.out.println(error.getData().get("Recommend"));
            com.aliyun.teautil.Common.assertAsString(error.message);
        }
    }

    /**
     * 查询实例列表
     * @param regionId
     * @return
     * @throws Exception
     */
    public static List<String>  FindInstance( String regionId) throws Exception {
        // 创建ECS客户端
        Client client = ECSTool.createClient ( );
        DescribeInstancesRequest describeInstancesRequest = new DescribeInstancesRequest ( )
                .setRegionId ( regionId );
        RuntimeOptions runtime = new RuntimeOptions ( );
        List< String > ipList = new ArrayList<> ( );
        try {
            // 复制代码运行请自行打印 API 的返回值
            DescribeInstancesResponse response = client.describeInstancesWithOptions ( describeInstancesRequest, runtime );

            List< List< String > > ipListList = response.getBody ( ).instances.getInstance ( ).stream ( ).map ( instance -> instance.publicIpAddress.ipAddress ).collect ( Collectors.toList ( ) );
            for (List< String > strings : ipListList) {
                for (String ip : strings) {
                    ipList.add ( ip );
                }
                System.out.println ( "------------------------" );
            }
            log.info ( "ipList: " + ipList );  //  [39.100.89.218, 39.100.113.246]
            return ipList;
        } catch (TeaException error) {
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println ( error.getMessage ( ) );
            // 诊断地址
            System.out.println ( error.getData ( ).get ( "Recommend" ) );
            com.aliyun.teautil.Common.assertAsString ( error.message );
        } catch (Exception _error) {
            TeaException error = new TeaException ( _error.getMessage ( ), _error );
            // 此处仅做打印展示，请谨慎对待异常处理，在工程项目中切勿直接忽略异常。
            // 错误 message
            System.out.println ( error.getMessage ( ) );
            // 诊断地址
            System.out.println ( error.getData ( ).get ( "Recommend" ) );
            com.aliyun.teautil.Common.assertAsString ( error.message );
        }
        return null;
    }

}
